# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .aws_provider_credential import AwsProviderCredential
from .aws_region import AwsRegion
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .git_lab_credential import GitLabCredential
from .hcl_app_scan_on_cloud_credential import HclAppScanOnCloudCredential
from .hcl_app_scan_on_cloud_url import HclAppScanOnCloudUrl
from .open_text_application_security_credential import (
    OpenTextApplicationSecurityCredential,
)
from .open_text_core_application_security_credential import (
    OpenTextCoreApplicationSecurityCredential,
)
from .open_text_core_application_security_url import OpenTextCoreApplicationSecurityUrl
from .appsec_opentext_core_application_security_dataset import (
    AppsecOpentextCoreApplicationSecurityDataset,
)
from .armis_credential import ArmisCredential
from .assets_armis_dataset import AssetsArmisDataset
from .axonius_credential import AxoniusCredential
from .assets_axonius_dataset import AssetsAxoniusDataset
from .claroty_credential import ClarotyCredential
from .claroty_apiurl import ClarotyApiurl
from .crowd_strike_credential import CrowdStrikeCredential
from .assets_crowd_strike_dataset import AssetsCrowdStrikeDataset
from .ivanti_credential import IvantiCredential
from .nozomi_vantage_credential import NozomiVantageCredential
from .assets_nozomi_vantage_dataset import AssetsNozomiVantageDataset
from .qualys_cloud_credential import QualysCloudCredential
from .assets_qualys_cloud_dataset import AssetsQualysCloudDataset
from .service_now_credential import ServiceNowCredential
from .assets_service_now_dataset import AssetsServiceNowDataset
from .sevco_credential import SevcoCredential
from .assets_sevco_dataset import AssetsSevcoDataset
from .tanium_cloud_credential import TaniumCloudCredential
from .assets_tanium_cloud_dataset import AssetsTaniumCloudDataset
from .defender_credential import DefenderCredential
from .palo_alto_credential import PaloAltoCredential
from .custom_endpoint import CustomEndpoint
from .edr_crowd_strike_dataset import EdrCrowdStrikeDataset
from .malwarebytes_credential import MalwarebytesCredential
from .sentinel_one_credential import SentinelOneCredential
from .sentinel_one_edr_events_credential import SentinelOneEdrEventsCredential
from .sophos_credential import SophosCredential
from .entra_id_credential import EntraIdCredential
from .google_credential import GoogleCredential
from .okta_credential import OktaCredential
from .ping_one_auth_url import PingOneAuthUrl
from .ping_one_credential import PingOneCredential
from .ping_one_apiurl import PingOneApiurl
from .incident_io_credential import IncidentIoCredential
from .pager_duty_credential import PagerDutyCredential
from .jira_credential import JiraCredential
from .slack_credential import SlackCredential
from .slack_webhook_credential import SlackWebhookCredential
from .teams_credential import TeamsCredential
from .crowdstrike_hec_credential import CrowdstrikeHecCredential
from .elasticsearch_auth_options import ElasticsearchAuthOptions
from .elasticsearch_credential import ElasticsearchCredential
from .google_chronicle_credential import GoogleChronicleCredential
from .google_service_account_credential import GoogleServiceAccountCredential
from .open_search_credential import OpenSearchCredential
from .q_radar_credential import QRadarCredential
from .rapid_7_insight_cloud_credential import Rapid7InsightCloudCredential
from .sentinel_credential import SentinelCredential
from .splunk_hec_token import SplunkHecToken
from .splunk_search_credential import SplunkSearchCredential
from .sumo_logic_collection_url import SumoLogicCollectionUrl
from .sumo_logic_credential import SumoLogicCredential
from .aws_s_3_credential import AwsS3Credential
from .aws_security_lake_credential import AwsSecurityLakeCredential
from .aws_sqs_credential import AwsSqsCredential
from .azure_blob_credential import AzureBlobCredential
from .azure_monitor_logs_credential import AzureMonitorLogsCredential
from .gcs_json_credential import GcsJsonCredential
from .gcs_credential import GcsCredential
from .autotask_api_integration_code_credential import (
    AutotaskApiIntegrationCodeCredential,
)
from .autotask_secret_credential import AutotaskSecretCredential
from .freshdesk_credential import FreshdeskCredential
from .ivanti_credential_ticketing import IvantiCredentialTicketing
from .custom_field_mapping import CustomFieldMapping
from .ticketing_pagerduty_dataset import TicketingPagerdutyDataset
from .torq_credential import TorqCredential
from .zendesk_credential import ZendeskCredential
from .vulnerabilities_crowd_strike_dataset import VulnerabilitiesCrowdStrikeDataset
from .nucleus_credential import NucleusCredential
from .vulnerabilities_qualys_cloud_dataset import VulnerabilitiesQualysCloudDataset
from .vulnerabilities_rapid_7_insight_cloud_dataset import (
    VulnerabilitiesRapid7InsightCloudDataset,
)
from .vulnerabilities_tanium_cloud_dataset import VulnerabilitiesTaniumCloudDataset
from .tenable_cloud_credential import TenableCloudCredential
import typing_extensions
from ...core.unchecked_base_model import UnionMetadata


class ProviderConfig_AppsecAmazonInspector(UncheckedBaseModel):
    type: typing.Literal["appsec_amazon_inspector"] = "appsec_amazon_inspector"
    credential: AwsProviderCredential
    region: AwsRegion

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AppsecGitlab(UncheckedBaseModel):
    type: typing.Literal["appsec_gitlab"] = "appsec_gitlab"
    credential: GitLabCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AppsecHclAppscanOnCloud(UncheckedBaseModel):
    type: typing.Literal["appsec_hcl_appscan_on_cloud"] = "appsec_hcl_appscan_on_cloud"
    credential: HclAppScanOnCloudCredential
    url: HclAppScanOnCloudUrl

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AppsecOpentextApplicationSecurity(UncheckedBaseModel):
    type: typing.Literal["appsec_opentext_application_security"] = (
        "appsec_opentext_application_security"
    )
    credential: OpenTextApplicationSecurityCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AppsecOpentextCoreApplicationSecurity(UncheckedBaseModel):
    type: typing.Literal["appsec_opentext_core_application_security"] = (
        "appsec_opentext_core_application_security"
    )
    credential: OpenTextCoreApplicationSecurityCredential
    url: OpenTextCoreApplicationSecurityUrl

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AppsecOpentextCoreApplicationSecurityMock(UncheckedBaseModel):
    type: typing.Literal["appsec_opentext_core_application_security_mock"] = (
        "appsec_opentext_core_application_security_mock"
    )
    dataset: AppsecOpentextCoreApplicationSecurityDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsArmisCentrix(UncheckedBaseModel):
    type: typing.Literal["assets_armis_centrix"] = "assets_armis_centrix"
    credential: ArmisCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsArmisCentrixMock(UncheckedBaseModel):
    type: typing.Literal["assets_armis_centrix_mock"] = "assets_armis_centrix_mock"
    dataset: AssetsArmisDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsAxonius(UncheckedBaseModel):
    type: typing.Literal["assets_axonius"] = "assets_axonius"
    credential: AxoniusCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsAxoniusMock(UncheckedBaseModel):
    type: typing.Literal["assets_axonius_mock"] = "assets_axonius_mock"
    dataset: AssetsAxoniusDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsClarotyXdome(UncheckedBaseModel):
    type: typing.Literal["assets_claroty_xdome"] = "assets_claroty_xdome"
    credential: ClarotyCredential
    url: ClarotyApiurl

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsCrowdstrike(UncheckedBaseModel):
    type: typing.Literal["assets_crowdstrike"] = "assets_crowdstrike"
    credential: CrowdStrikeCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsCrowdstrikeMock(UncheckedBaseModel):
    type: typing.Literal["assets_crowdstrike_mock"] = "assets_crowdstrike_mock"
    dataset: AssetsCrowdStrikeDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsIvantiNeurons(UncheckedBaseModel):
    type: typing.Literal["assets_ivanti_neurons"] = "assets_ivanti_neurons"
    account_identifier: str
    credential: IvantiCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsNozomiVantage(UncheckedBaseModel):
    type: typing.Literal["assets_nozomi_vantage"] = "assets_nozomi_vantage"
    credential: NozomiVantageCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsNozomiVantageMock(UncheckedBaseModel):
    type: typing.Literal["assets_nozomi_vantage_mock"] = "assets_nozomi_vantage_mock"
    dataset: AssetsNozomiVantageDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsQualysCloud(UncheckedBaseModel):
    type: typing.Literal["assets_qualys_cloud"] = "assets_qualys_cloud"
    credential: QualysCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsQualysCloudMock(UncheckedBaseModel):
    type: typing.Literal["assets_qualys_cloud_mock"] = "assets_qualys_cloud_mock"
    dataset: AssetsQualysCloudDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsServicenow(UncheckedBaseModel):
    type: typing.Literal["assets_servicenow"] = "assets_servicenow"
    credential: ServiceNowCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsServicenowMock(UncheckedBaseModel):
    type: typing.Literal["assets_servicenow_mock"] = "assets_servicenow_mock"
    dataset: AssetsServiceNowDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsSevco(UncheckedBaseModel):
    type: typing.Literal["assets_sevco"] = "assets_sevco"
    credential: SevcoCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsSevcoMock(UncheckedBaseModel):
    type: typing.Literal["assets_sevco_mock"] = "assets_sevco_mock"
    dataset: AssetsSevcoDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsTaniumCloud(UncheckedBaseModel):
    type: typing.Literal["assets_tanium_cloud"] = "assets_tanium_cloud"
    credential: TaniumCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_AssetsTaniumCloudMock(UncheckedBaseModel):
    type: typing.Literal["assets_tanium_cloud_mock"] = "assets_tanium_cloud_mock"
    dataset: AssetsTaniumCloudDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_CloudsecurityAws(UncheckedBaseModel):
    type: typing.Literal["cloudsecurity_aws"] = "cloudsecurity_aws"
    credential: AwsProviderCredential
    region: AwsRegion

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_CloudsecurityCrowdstrike(UncheckedBaseModel):
    type: typing.Literal["cloudsecurity_crowdstrike"] = "cloudsecurity_crowdstrike"
    credential: CrowdStrikeCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_CloudsecurityDefender(UncheckedBaseModel):
    type: typing.Literal["cloudsecurity_defender"] = "cloudsecurity_defender"
    credential: DefenderCredential
    subscription_id: str
    tenant_id: str
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_CloudsecurityPaloalto(UncheckedBaseModel):
    type: typing.Literal["cloudsecurity_paloalto"] = "cloudsecurity_paloalto"
    credential: PaloAltoCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_CustomSynqly(UncheckedBaseModel):
    type: typing.Literal["custom_synqly"] = "custom_synqly"
    custom_id: str
    endpoints: typing.Dict[str, CustomEndpoint]
    tags: typing.Dict[str, str]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrCrowdstrike(UncheckedBaseModel):
    type: typing.Literal["edr_crowdstrike"] = "edr_crowdstrike"
    credential: CrowdStrikeCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrCrowdstrikeMock(UncheckedBaseModel):
    type: typing.Literal["edr_crowdstrike_mock"] = "edr_crowdstrike_mock"
    dataset: EdrCrowdStrikeDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrDefender(UncheckedBaseModel):
    type: typing.Literal["edr_defender"] = "edr_defender"
    credential: DefenderCredential
    tenant_id: str
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrMalwarebytes(UncheckedBaseModel):
    type: typing.Literal["edr_malwarebytes"] = "edr_malwarebytes"
    account_identifier: str
    credential: MalwarebytesCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrSentinelone(UncheckedBaseModel):
    type: typing.Literal["edr_sentinelone"] = "edr_sentinelone"
    credential: SentinelOneCredential
    edr_events_credential: typing.Optional[SentinelOneEdrEventsCredential] = None
    edr_events_url: typing.Optional[str] = None
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrSophos(UncheckedBaseModel):
    type: typing.Literal["edr_sophos"] = "edr_sophos"
    credential: SophosCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_EdrTanium(UncheckedBaseModel):
    type: typing.Literal["edr_tanium"] = "edr_tanium"
    credential: TaniumCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IdentityEntraId(UncheckedBaseModel):
    type: typing.Literal["identity_entra_id"] = "identity_entra_id"
    credential: EntraIdCredential
    tenant_id: str
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IdentityGoogle(UncheckedBaseModel):
    type: typing.Literal["identity_google"] = "identity_google"
    client_email: str
    credential: GoogleCredential
    delegate: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IdentityOkta(UncheckedBaseModel):
    type: typing.Literal["identity_okta"] = "identity_okta"
    credential: OktaCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IdentityPingone(UncheckedBaseModel):
    type: typing.Literal["identity_pingone"] = "identity_pingone"
    auth_url: PingOneAuthUrl
    client_id: str
    credential: PingOneCredential
    organization_id: str
    url: PingOneApiurl

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IncidentresponseIncidentio(UncheckedBaseModel):
    type: typing.Literal["incidentresponse_incidentio"] = "incidentresponse_incidentio"
    credential: IncidentIoCredential

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_IncidentresponsePagerduty(UncheckedBaseModel):
    type: typing.Literal["incidentresponse_pagerduty"] = "incidentresponse_pagerduty"
    credential: PagerDutyCredential

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_NotificationsJira(UncheckedBaseModel):
    type: typing.Literal["notifications_jira"] = "notifications_jira"
    credential: JiraCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_NotificationsMockNotifications(UncheckedBaseModel):
    type: typing.Literal["notifications_mock_notifications"] = (
        "notifications_mock_notifications"
    )
    channel: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_NotificationsSlack(UncheckedBaseModel):
    type: typing.Literal["notifications_slack"] = "notifications_slack"
    channel: str
    credential: SlackCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_NotificationsSlackWebhook(UncheckedBaseModel):
    type: typing.Literal["notifications_slack_webhook"] = "notifications_slack_webhook"
    webhook_url: SlackWebhookCredential

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_NotificationsTeams(UncheckedBaseModel):
    type: typing.Literal["notifications_teams"] = "notifications_teams"
    channel_id: str
    credential: TeamsCredential
    endpoint: typing.Optional[str] = None
    team_id: str
    tenant_id: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemCrowdstrike(UncheckedBaseModel):
    type: typing.Literal["siem_crowdstrike"] = "siem_crowdstrike"
    credential: CrowdStrikeCredential
    hec_credential: typing.Optional[CrowdstrikeHecCredential] = None
    hec_url: typing.Optional[str] = None
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemElasticsearch(UncheckedBaseModel):
    type: typing.Literal["siem_elasticsearch"] = "siem_elasticsearch"
    auth_options: typing.Optional[ElasticsearchAuthOptions] = None
    create_index: typing.Optional[str] = None
    credential: ElasticsearchCredential
    index: typing.Optional[str] = None
    kibana_url: typing.Optional[str] = None
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemGoogleChronicle(UncheckedBaseModel):
    type: typing.Literal["siem_google_chronicle"] = "siem_google_chronicle"
    customer_id: typing.Optional[str] = None
    ingestion_credential: typing.Optional[GoogleChronicleCredential] = None
    ingestion_url: typing.Optional[str] = None
    search_credential: GoogleChronicleCredential
    search_url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemGoogleSecurityOperations(UncheckedBaseModel):
    type: typing.Literal["siem_google_security_operations"] = (
        "siem_google_security_operations"
    )
    credential: GoogleServiceAccountCredential
    customer_id: str
    project_id: str
    region: typing.Optional[str] = None
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemMockSiem(UncheckedBaseModel):
    type: typing.Literal["siem_mock_siem"] = "siem_mock_siem"
    index: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemOpensearch(UncheckedBaseModel):
    type: typing.Literal["siem_opensearch"] = "siem_opensearch"
    create_index: typing.Optional[str] = None
    credential: OpenSearchCredential
    index: typing.Optional[str] = None
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemQRadar(UncheckedBaseModel):
    type: typing.Literal["siem_q_radar"] = "siem_q_radar"
    collection_port: int
    credential: QRadarCredential
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemRapid7Insightidr(UncheckedBaseModel):
    type: typing.Literal["siem_rapid7_insightidr"] = "siem_rapid7_insightidr"
    credential: Rapid7InsightCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemSentinel(UncheckedBaseModel):
    type: typing.Literal["siem_sentinel"] = "siem_sentinel"
    credential: SentinelCredential
    default_tables: typing.Optional[typing.List[str]] = None
    ingest_url: typing.Optional[str] = None
    logs_url: typing.Optional[str] = None
    management_url: typing.Optional[str] = None
    resource_group: str
    rule_id: typing.Optional[str] = None
    stream_name: typing.Optional[str] = None
    subscription_id: str
    tenant_id: str
    workspace_id: str
    workspace_name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemSplunk(UncheckedBaseModel):
    type: typing.Literal["siem_splunk"] = "siem_splunk"
    hec_credential: typing.Optional[SplunkHecToken] = None
    hec_url: typing.Optional[str] = None
    index: typing.Optional[str] = None
    search_service_credential: SplunkSearchCredential
    search_service_url: str
    skip_tls_verify: typing.Optional[bool] = None
    source: typing.Optional[str] = None
    source_type: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SiemSumoLogic(UncheckedBaseModel):
    type: typing.Literal["siem_sumo_logic"] = "siem_sumo_logic"
    auto_parse_logs: typing.Optional[bool] = None
    collection_url: typing.Optional[SumoLogicCollectionUrl] = None
    credential: SumoLogicCredential
    siem_logs_only: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkAwsS3(UncheckedBaseModel):
    type: typing.Literal["sink_aws_s3"] = "sink_aws_s3"
    bucket: str
    credential: AwsS3Credential
    path: str
    region: AwsRegion

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkAwsSecurityLake(UncheckedBaseModel):
    type: typing.Literal["sink_aws_security_lake"] = "sink_aws_security_lake"
    credential: AwsSecurityLakeCredential
    region: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkAwsSqs(UncheckedBaseModel):
    type: typing.Literal["sink_aws_sqs"] = "sink_aws_sqs"
    credential: AwsSqsCredential
    region: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkAzureBlob(UncheckedBaseModel):
    type: typing.Literal["sink_azure_blob"] = "sink_azure_blob"
    bucket: str
    credential: AzureBlobCredential
    path: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkAzureMonitorLogs(UncheckedBaseModel):
    type: typing.Literal["sink_azure_monitor_logs"] = "sink_azure_monitor_logs"
    client_id: str
    credential: AzureMonitorLogsCredential
    rule_id: str
    stream_name: str
    tenant_id: str
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkCrowdstrikeHec(UncheckedBaseModel):
    type: typing.Literal["sink_crowdstrike_hec"] = "sink_crowdstrike_hec"
    credential: CrowdstrikeHecCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkElasticsearch(UncheckedBaseModel):
    type: typing.Literal["sink_elasticsearch"] = "sink_elasticsearch"
    auth_options: typing.Optional[ElasticsearchAuthOptions] = None
    create_index: str
    credential: ElasticsearchCredential
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkGcs(UncheckedBaseModel):
    type: typing.Literal["sink_gcs"] = "sink_gcs"
    bucket: str
    credential: GcsJsonCredential
    path: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkGoogleSecOps(UncheckedBaseModel):
    type: typing.Literal["sink_google_sec_ops"] = "sink_google_sec_ops"
    credential: GoogleChronicleCredential
    customer_id: str
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkGoogleSecurityOperations(UncheckedBaseModel):
    type: typing.Literal["sink_google_security_operations"] = (
        "sink_google_security_operations"
    )
    credential: GoogleServiceAccountCredential
    customer_id: str
    project_id: str
    region: typing.Optional[str] = None
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkMockSink(UncheckedBaseModel):
    type: typing.Literal["sink_mock_sink"] = "sink_mock_sink"
    destination: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkOpensearch(UncheckedBaseModel):
    type: typing.Literal["sink_opensearch"] = "sink_opensearch"
    create_index: str
    credential: OpenSearchCredential
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkQRadar(UncheckedBaseModel):
    type: typing.Literal["sink_q_radar"] = "sink_q_radar"
    collection_port: int
    skip_tls_verify: typing.Optional[bool] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkSplunk(UncheckedBaseModel):
    type: typing.Literal["sink_splunk"] = "sink_splunk"
    hec_credential: SplunkHecToken
    hec_url: str
    index: typing.Optional[str] = None
    skip_tls_verify: typing.Optional[bool] = None
    source: typing.Optional[str] = None
    source_type: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_SinkSumoLogic(UncheckedBaseModel):
    type: typing.Literal["sink_sumo_logic"] = "sink_sumo_logic"
    collection_url: SumoLogicCollectionUrl

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_StorageAwsS3(UncheckedBaseModel):
    type: typing.Literal["storage_aws_s3"] = "storage_aws_s3"
    bucket: str
    credential: AwsS3Credential
    endpoint: typing.Optional[str] = None
    region: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_StorageAzureBlob(UncheckedBaseModel):
    type: typing.Literal["storage_azure_blob"] = "storage_azure_blob"
    bucket: str
    credential: AzureBlobCredential

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_StorageGcs(UncheckedBaseModel):
    type: typing.Literal["storage_gcs"] = "storage_gcs"
    bucket: str
    credential: GcsCredential
    region: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_StorageMockStorage(UncheckedBaseModel):
    type: typing.Literal["storage_mock_storage"] = "storage_mock_storage"
    bucket: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingAutotask(UncheckedBaseModel):
    type: typing.Literal["ticketing_autotask"] = "ticketing_autotask"
    api_integration_code_credential: AutotaskApiIntegrationCodeCredential
    default_issue_type: typing.Optional[int] = None
    default_project: typing.Optional[int] = None
    secret_credential: AutotaskSecretCredential
    user_name: str
    zone_path: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingFreshdesk(UncheckedBaseModel):
    type: typing.Literal["ticketing_freshdesk"] = "ticketing_freshdesk"
    credential: FreshdeskCredential
    default_contact: typing.Optional[str] = None
    default_type: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingIvanti(UncheckedBaseModel):
    type: typing.Literal["ticketing_ivanti"] = "ticketing_ivanti"
    credential: IvantiCredentialTicketing
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingJira(UncheckedBaseModel):
    type: typing.Literal["ticketing_jira"] = "ticketing_jira"
    credential: JiraCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    default_issue_type: typing.Optional[str] = None
    default_project: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingJiraServiceManagement(UncheckedBaseModel):
    type: typing.Literal["ticketing_jira_service_management"] = (
        "ticketing_jira_service_management"
    )
    credential: JiraCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    default_issue_type: typing.Optional[str] = None
    default_project: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingMockTicketing(UncheckedBaseModel):
    type: typing.Literal["ticketing_mock_ticketing"] = "ticketing_mock_ticketing"
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    name: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingPagerduty(UncheckedBaseModel):
    type: typing.Literal["ticketing_pagerduty"] = "ticketing_pagerduty"
    credential: PagerDutyCredential
    default_creator: typing.Optional[str] = None
    default_project: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingPagerdutyMock(UncheckedBaseModel):
    type: typing.Literal["ticketing_pagerduty_mock"] = "ticketing_pagerduty_mock"
    dataset: TicketingPagerdutyDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingServicenow(UncheckedBaseModel):
    type: typing.Literal["ticketing_servicenow"] = "ticketing_servicenow"
    credential: ServiceNowCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    default_issue_type: typing.Optional[str] = None
    default_project: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingServicenowSir(UncheckedBaseModel):
    type: typing.Literal["ticketing_servicenow_sir"] = "ticketing_servicenow_sir"
    credential: ServiceNowCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    default_creator: typing.Optional[str] = None
    default_issue_type: typing.Optional[str] = None
    default_project: typing.Optional[str] = None
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingTorq(UncheckedBaseModel):
    type: typing.Literal["ticketing_torq"] = "ticketing_torq"
    credential: TorqCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = None
    default_creator: typing.Optional[str] = None
    default_issue_type: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_TicketingZendesk(UncheckedBaseModel):
    type: typing.Literal["ticketing_zendesk"] = "ticketing_zendesk"
    credential: ZendeskCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesCrowdstrike(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_crowdstrike"] = "vulnerabilities_crowdstrike"
    credential: CrowdStrikeCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesCrowdstrikeMock(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_crowdstrike_mock"] = (
        "vulnerabilities_crowdstrike_mock"
    )
    dataset: VulnerabilitiesCrowdStrikeDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesNucleus(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_nucleus"] = "vulnerabilities_nucleus"
    credential: NucleusCredential
    project_id: str
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesQualysCloud(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_qualys_cloud"] = (
        "vulnerabilities_qualys_cloud"
    )
    credential: QualysCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesQualysCloudMock(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_qualys_cloud_mock"] = (
        "vulnerabilities_qualys_cloud_mock"
    )
    dataset: VulnerabilitiesQualysCloudDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesRapid7InsightCloud(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_rapid7_insight_cloud"] = (
        "vulnerabilities_rapid7_insight_cloud"
    )
    credential: Rapid7InsightCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesRapid7InsightCloudMock(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_rapid7_insight_cloud_mock"] = (
        "vulnerabilities_rapid7_insight_cloud_mock"
    )
    dataset: VulnerabilitiesRapid7InsightCloudDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesServicenowVr(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_servicenow_vr"] = (
        "vulnerabilities_servicenow_vr"
    )
    credential: ServiceNowCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesTaniumCloud(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_tanium_cloud"] = (
        "vulnerabilities_tanium_cloud"
    )
    credential: TaniumCloudCredential
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesTaniumCloudMock(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_tanium_cloud_mock"] = (
        "vulnerabilities_tanium_cloud_mock"
    )
    dataset: VulnerabilitiesTaniumCloudDataset

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


class ProviderConfig_VulnerabilitiesTenableCloud(UncheckedBaseModel):
    type: typing.Literal["vulnerabilities_tenable_cloud"] = (
        "vulnerabilities_tenable_cloud"
    )
    credential: TenableCloudCredential
    url: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


ProviderConfig = typing_extensions.Annotated[
    typing.Union[
        ProviderConfig_AppsecAmazonInspector,
        ProviderConfig_AppsecGitlab,
        ProviderConfig_AppsecHclAppscanOnCloud,
        ProviderConfig_AppsecOpentextApplicationSecurity,
        ProviderConfig_AppsecOpentextCoreApplicationSecurity,
        ProviderConfig_AppsecOpentextCoreApplicationSecurityMock,
        ProviderConfig_AssetsArmisCentrix,
        ProviderConfig_AssetsArmisCentrixMock,
        ProviderConfig_AssetsAxonius,
        ProviderConfig_AssetsAxoniusMock,
        ProviderConfig_AssetsClarotyXdome,
        ProviderConfig_AssetsCrowdstrike,
        ProviderConfig_AssetsCrowdstrikeMock,
        ProviderConfig_AssetsIvantiNeurons,
        ProviderConfig_AssetsNozomiVantage,
        ProviderConfig_AssetsNozomiVantageMock,
        ProviderConfig_AssetsQualysCloud,
        ProviderConfig_AssetsQualysCloudMock,
        ProviderConfig_AssetsServicenow,
        ProviderConfig_AssetsServicenowMock,
        ProviderConfig_AssetsSevco,
        ProviderConfig_AssetsSevcoMock,
        ProviderConfig_AssetsTaniumCloud,
        ProviderConfig_AssetsTaniumCloudMock,
        ProviderConfig_CloudsecurityAws,
        ProviderConfig_CloudsecurityCrowdstrike,
        ProviderConfig_CloudsecurityDefender,
        ProviderConfig_CloudsecurityPaloalto,
        ProviderConfig_CustomSynqly,
        ProviderConfig_EdrCrowdstrike,
        ProviderConfig_EdrCrowdstrikeMock,
        ProviderConfig_EdrDefender,
        ProviderConfig_EdrMalwarebytes,
        ProviderConfig_EdrSentinelone,
        ProviderConfig_EdrSophos,
        ProviderConfig_EdrTanium,
        ProviderConfig_IdentityEntraId,
        ProviderConfig_IdentityGoogle,
        ProviderConfig_IdentityOkta,
        ProviderConfig_IdentityPingone,
        ProviderConfig_IncidentresponseIncidentio,
        ProviderConfig_IncidentresponsePagerduty,
        ProviderConfig_NotificationsJira,
        ProviderConfig_NotificationsMockNotifications,
        ProviderConfig_NotificationsSlack,
        ProviderConfig_NotificationsSlackWebhook,
        ProviderConfig_NotificationsTeams,
        ProviderConfig_SiemCrowdstrike,
        ProviderConfig_SiemElasticsearch,
        ProviderConfig_SiemGoogleChronicle,
        ProviderConfig_SiemGoogleSecurityOperations,
        ProviderConfig_SiemMockSiem,
        ProviderConfig_SiemOpensearch,
        ProviderConfig_SiemQRadar,
        ProviderConfig_SiemRapid7Insightidr,
        ProviderConfig_SiemSentinel,
        ProviderConfig_SiemSplunk,
        ProviderConfig_SiemSumoLogic,
        ProviderConfig_SinkAwsS3,
        ProviderConfig_SinkAwsSecurityLake,
        ProviderConfig_SinkAwsSqs,
        ProviderConfig_SinkAzureBlob,
        ProviderConfig_SinkAzureMonitorLogs,
        ProviderConfig_SinkCrowdstrikeHec,
        ProviderConfig_SinkElasticsearch,
        ProviderConfig_SinkGcs,
        ProviderConfig_SinkGoogleSecOps,
        ProviderConfig_SinkGoogleSecurityOperations,
        ProviderConfig_SinkMockSink,
        ProviderConfig_SinkOpensearch,
        ProviderConfig_SinkQRadar,
        ProviderConfig_SinkSplunk,
        ProviderConfig_SinkSumoLogic,
        ProviderConfig_StorageAwsS3,
        ProviderConfig_StorageAzureBlob,
        ProviderConfig_StorageGcs,
        ProviderConfig_StorageMockStorage,
        ProviderConfig_TicketingAutotask,
        ProviderConfig_TicketingFreshdesk,
        ProviderConfig_TicketingIvanti,
        ProviderConfig_TicketingJira,
        ProviderConfig_TicketingJiraServiceManagement,
        ProviderConfig_TicketingMockTicketing,
        ProviderConfig_TicketingPagerduty,
        ProviderConfig_TicketingPagerdutyMock,
        ProviderConfig_TicketingServicenow,
        ProviderConfig_TicketingServicenowSir,
        ProviderConfig_TicketingTorq,
        ProviderConfig_TicketingZendesk,
        ProviderConfig_VulnerabilitiesCrowdstrike,
        ProviderConfig_VulnerabilitiesCrowdstrikeMock,
        ProviderConfig_VulnerabilitiesNucleus,
        ProviderConfig_VulnerabilitiesQualysCloud,
        ProviderConfig_VulnerabilitiesQualysCloudMock,
        ProviderConfig_VulnerabilitiesRapid7InsightCloud,
        ProviderConfig_VulnerabilitiesRapid7InsightCloudMock,
        ProviderConfig_VulnerabilitiesServicenowVr,
        ProviderConfig_VulnerabilitiesTaniumCloud,
        ProviderConfig_VulnerabilitiesTaniumCloudMock,
        ProviderConfig_VulnerabilitiesTenableCloud,
    ],
    UnionMetadata(discriminant="type"),
]



# This file was auto-generated by Fern from our API Definition.

from ......core.unchecked_base_model import UncheckedBaseModel
import typing
from .d_3_fend import D3Fend
import pydantic
from ...base.types.url_string import UrlString
from ......core.pydantic_utilities import IS_PYDANTIC_V2


class Mitigation(UncheckedBaseModel):
    """
    The MITRE Mitigation object describes the ATT&CK® or ATLAS™ Mitigation ID and/or name that is associated to an attack.
    """

    countermeasures: typing.Optional[typing.List[D3Fend]] = pydantic.Field(default=None)
    """
    The D3FEND countermeasures that are associated with the attack technique. For example: ATT&CK Technique <code>T1003</code> is addressed by Mitigation <code>M1027</code>, and D3FEND Technique <code>D3-OTP</code>.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Mitigation name that is associated with the attack technique. For example: <code>Password Policies</code>, or <code>Code Signing</code>.
    """

    src_url: typing.Optional[UrlString] = pydantic.Field(default=None)
    """
    The versioned permalink of the Mitigation. For example: <code>https://attack.mitre.org/versions/v14/mitigations/M1027</code>.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The Mitigation ID that is associated with the attack technique. For example: <code>M1027</code>, or <code>AML.M0013</code>.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .account import Account
from .account_type_id import AccountTypeId
from .actor import Actor
from .actor_actor_type_id import ActorActorTypeId
from .affected_code import AffectedCode
from .affected_package import AffectedPackage
from .affected_package_type_id import AffectedPackageTypeId
from .agent import Agent
from .agent_type_id import AgentTypeId
from .analytic import Analytic
from .analytic_type_id import AnalyticTypeId
from .api import Api
from .attack import Attack
from .authorization import Authorization
from .autonomous_system import AutonomousSystem
from .certificate import Certificate
from .cloud import Cloud
from .compliance import Compliance
from .compliance_status_id import ComplianceStatusId
from .container import Container
from .cve import Cve
from .cvss import Cvss
from .cvss_depth import CvssDepth
from .cwe import Cwe
from .device import Device
from .device_hw_info import DeviceHwInfo
from .device_network_status_id import DeviceNetworkStatusId
from .device_risk_level_id import DeviceRiskLevelId
from .device_type_id import DeviceTypeId
from .digital_signature import DigitalSignature
from .digital_signature_algorithm_id import DigitalSignatureAlgorithmId
from .digital_signature_state_id import DigitalSignatureStateId
from .display import Display
from .dns_answer import DnsAnswer
from .dns_answer_flag_ids import DnsAnswerFlagIds
from .domain_contact import DomainContact
from .domain_contact_type_id import DomainContactTypeId
from .email import Email
from .email_auth import EmailAuth
from .enrichment import Enrichment
from .epss import Epss
from .extension import Extension
from .feature import Feature
from .file import File
from .file_confidentiality_id import FileConfidentialityId
from .file_type_id import FileTypeId
from .finding_info import FindingInfo
from .fingerprint import Fingerprint
from .fingerprint_algorithm_id import FingerprintAlgorithmId
from .group import Group
from .idp import Idp
from .image import Image
from .kb_article import KbArticle
from .kb_article_install_state_id import KbArticleInstallStateId
from .keyboard_info import KeyboardInfo
from .kill_chain_phase import KillChainPhase
from .kill_chain_phase_phase_id import KillChainPhasePhaseId
from .ldap_person import LdapPerson
from .location import Location
from .logger import Logger
from .metadata import Metadata
from .metric import Metric
from .network_interface import NetworkInterface
from .network_interface_type_id import NetworkInterfaceTypeId
from .object import Object
from .observable import Observable
from .observable_type_id import ObservableTypeId
from .organization import Organization
from .os import Os
from .os_type_id import OsTypeId
from .osint import Osint
from .osint_confidence_id import OsintConfidenceId
from .osint_tlp import OsintTlp
from .osint_type_id import OsintTypeId
from .package import Package
from .package_type_id import PackageTypeId
from .policy import Policy
from .process import Process
from .process_integrity_id import ProcessIntegrityId
from .product import Product
from .related_event import RelatedEvent
from .remediation import Remediation
from .reputation import Reputation
from .reputation_score_id import ReputationScoreId
from .request import Request
from .resource_details import ResourceDetails
from .response import Response
from .service import Service
from .session import Session
from .sub_technique import SubTechnique
from .tactic import Tactic
from .technique import Technique
from .timespan import Timespan
from .timespan_type_id import TimespanTypeId
from .url import Url
from .url_category_ids import UrlCategoryIds
from .user import User
from .user_mfa_status_id import UserMfaStatusId
from .user_risk_level_id import UserRiskLevelId
from .user_type_id import UserTypeId
from .user_user_status_id import UserUserStatusId
from .vulnerability import Vulnerability
from .whois import Whois
from .whois_dnssec_status_id import WhoisDnssecStatusId

__all__ = [
    "Account",
    "AccountTypeId",
    "Actor",
    "ActorActorTypeId",
    "AffectedCode",
    "AffectedPackage",
    "AffectedPackageTypeId",
    "Agent",
    "AgentTypeId",
    "Analytic",
    "AnalyticTypeId",
    "Api",
    "Attack",
    "Authorization",
    "AutonomousSystem",
    "Certificate",
    "Cloud",
    "Compliance",
    "ComplianceStatusId",
    "Container",
    "Cve",
    "Cvss",
    "CvssDepth",
    "Cwe",
    "Device",
    "DeviceHwInfo",
    "DeviceNetworkStatusId",
    "DeviceRiskLevelId",
    "DeviceTypeId",
    "DigitalSignature",
    "DigitalSignatureAlgorithmId",
    "DigitalSignatureStateId",
    "Display",
    "DnsAnswer",
    "DnsAnswerFlagIds",
    "DomainContact",
    "DomainContactTypeId",
    "Email",
    "EmailAuth",
    "Enrichment",
    "Epss",
    "Extension",
    "Feature",
    "File",
    "FileConfidentialityId",
    "FileTypeId",
    "FindingInfo",
    "Fingerprint",
    "FingerprintAlgorithmId",
    "Group",
    "Idp",
    "Image",
    "KbArticle",
    "KbArticleInstallStateId",
    "KeyboardInfo",
    "KillChainPhase",
    "KillChainPhasePhaseId",
    "LdapPerson",
    "Location",
    "Logger",
    "Metadata",
    "Metric",
    "NetworkInterface",
    "NetworkInterfaceTypeId",
    "Object",
    "Observable",
    "ObservableTypeId",
    "Organization",
    "Os",
    "OsTypeId",
    "Osint",
    "OsintConfidenceId",
    "OsintTlp",
    "OsintTypeId",
    "Package",
    "PackageTypeId",
    "Policy",
    "Process",
    "ProcessIntegrityId",
    "Product",
    "RelatedEvent",
    "Remediation",
    "Reputation",
    "ReputationScoreId",
    "Request",
    "ResourceDetails",
    "Response",
    "Service",
    "Session",
    "SubTechnique",
    "Tactic",
    "Technique",
    "Timespan",
    "TimespanTypeId",
    "Url",
    "UrlCategoryIds",
    "User",
    "UserMfaStatusId",
    "UserRiskLevelId",
    "UserTypeId",
    "UserUserStatusId",
    "Vulnerability",
    "Whois",
    "WhoisDnssecStatusId",
]



# This file was auto-generated by Fern from our API Definition.

from .account_change import AccountChange
from .activity_id import ActivityId
from .category_uid import CategoryUid
from .class_uid import ClassUid
from .severity_id import SeverityId
from .status_id import StatusId
from .type_uid import TypeUid

__all__ = [
    "AccountChange",
    "ActivityId",
    "CategoryUid",
    "ClassUid",
    "SeverityId",
    "StatusId",
    "TypeUid",
]



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from .types.get_open_api_spec_response import GetOpenApiSpecResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.types.problem import Problem
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper


class MetaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def management_openapi_spec(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOpenApiSpecResponse:
        """
        Retrieve the OpenAPI spec for the Management API

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOpenApiSpecResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.meta.management_openapi_spec()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/meta/openapi/management",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOpenApiSpecResponse,
                    construct_type(
                        type_=GetOpenApiSpecResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def engine_openapi_spec(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOpenApiSpecResponse:
        """
        Retrieve the OpenAPI spec for the Engine APIs

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOpenApiSpecResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.meta.engine_openapi_spec()
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/meta/openapi/engine",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOpenApiSpecResponse,
                    construct_type(
                        type_=GetOpenApiSpecResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncMetaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def management_openapi_spec(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOpenApiSpecResponse:
        """
        Retrieve the OpenAPI spec for the Management API

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOpenApiSpecResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.meta.management_openapi_spec()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/meta/openapi/management",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOpenApiSpecResponse,
                    construct_type(
                        type_=GetOpenApiSpecResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def engine_openapi_spec(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetOpenApiSpecResponse:
        """
        Retrieve the OpenAPI spec for the Engine APIs

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetOpenApiSpecResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.meta.engine_openapi_spec()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/meta/openapi/engine",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    GetOpenApiSpecResponse,
                    construct_type(
                        type_=GetOpenApiSpecResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .jira_credential import JiraCredential
import typing
from .custom_field_mapping import CustomFieldMapping
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TicketingJiraServiceManagement(UncheckedBaseModel):
    """
    Configuration for Jira Service Management.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/jira-service-management-ticketing-setup)
    """

    credential: JiraCredential
    custom_field_mappings: typing.Optional[typing.List[CustomFieldMapping]] = (
        pydantic.Field(default=None)
    )
    """
    Custom field mappings for this provider.
    """

    default_issue_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default issue type when creating tickets.
    """

    default_project: typing.Optional[str] = pydantic.Field(default=None)
    """
    Default project when listing, creating, or editing tickets.
    """

    url: str = pydantic.Field()
    """
    Base URL for the Jira Service Management API.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
import typing
from ..core.request_options import RequestOptions
from .types.list_operations_response import ListOperationsResponse
from ..core.unchecked_base_model import construct_type
from ..common.errors.bad_request_error import BadRequestError
from ..common.types.problem import Problem
from ..common.errors.unauthorized_error import UnauthorizedError
from ..common.errors.forbidden_error import ForbiddenError
from ..common.errors.not_found_error import NotFoundError
from ..common.errors.method_not_allowed_error import MethodNotAllowedError
from ..common.errors.conflict_error import ConflictError
from ..common.errors.unsupported_media_type_error import UnsupportedMediaTypeError
from ..common.errors.too_many_requests_error import TooManyRequestsError
from ..common.errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..operation_history.types.list_execution_history_response import (
    ListExecutionHistoryResponse,
)
from ..core.client_wrapper import AsyncClientWrapper


class OperationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOperationsResponse:
        """
        Returns a list of all `Asynchronous Operations` objects.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Asynchronous Operations` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Asynchronous Operations` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `id`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field id. For example, `id[desc]` will sort the results by `id` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOperationsResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.operations.list(
            limit=1,
            start_after="string",
            order="string",
            filter="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/jobs",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListOperationsResponse,
                    construct_type(
                        type_=ListOperationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_execution_history(
        self,
        *,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListExecutionHistoryResponse:
        """
        Returns the execution history for all operations.
        History is stored for the configured retention period (default 4 weeks).

        Parameters
        ----------
        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter operation execution history by this query.
            Available filters:

            - `id[eq]` - Filter by specific operation ID
            - `integration_id[eq]` - Filter by specific integration ID
            - `account_id[eq]` - Filter by specific account ID
            - `execution_id[eq]` - Filter by specific execution ID
            - `started_at[gte]` - Filter executions started at or after a specific datetime (RFC3339 format)
            - `started_at[lte]` - Filter executions started at or before a specific datetime (RFC3339 format)
            - `started_at[gt]` - Filter executions started after a specific datetime
            - `started_at[lt]` - Filter executions started before a specific datetime
            - `operation_id[eq]` - Filter by operation name (e.g., "assets_query_devices")
              If used more than once, the queries are ANDed together.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `started_at[desc]`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field id. For example, `started_at[asc]` will sort the results by `started_at` in ascending order.
            The ordering defaults to `desc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        limit : typing.Optional[int]
            Number of results to return (default 100, max 500)

        start_after : typing.Optional[str]
            Return execution history starting after this cursor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListExecutionHistoryResponse

        Examples
        --------
        from synqly import SynqlyManagement

        client = SynqlyManagement(
            token="YOUR_TOKEN",
        )
        client.operations.list_execution_history(
            filter="string",
            order="string",
            limit=1,
            start_after="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/jobs/history",
            method="GET",
            params={
                "filter": filter,
                "order": order,
                "limit": limit,
                "start_after": start_after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListExecutionHistoryResponse,
                    construct_type(
                        type_=ListExecutionHistoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOperationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOperationsResponse:
        """
        Returns a list of all `Asynchronous Operations` objects.

        Parameters
        ----------
        limit : typing.Optional[int]
            Number of `Asynchronous Operations` objects to return in this page. Defaults to 100.

        start_after : typing.Optional[str]
            Return `Asynchronous Operations` objects starting after this `name`.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `id`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field id. For example, `id[desc]` will sort the results by `id` in descending order.
            The ordering defaults to `asc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter results by this query. For more information on filtering, refer to our Filtering Guide. Defaults to no filter.
            If used more than once, the queries are ANDed together.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOperationsResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.operations.list(
                limit=1,
                start_after="string",
                order="string",
                filter="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/jobs",
            method="GET",
            params={
                "limit": limit,
                "start_after": start_after,
                "order": order,
                "filter": filter,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListOperationsResponse,
                    construct_type(
                        type_=ListOperationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 405:
                raise MethodNotAllowedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 409:
                raise ConflictError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 415:
                raise UnsupportedMediaTypeError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_execution_history(
        self,
        *,
        filter: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        order: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        limit: typing.Optional[int] = None,
        start_after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListExecutionHistoryResponse:
        """
        Returns the execution history for all operations.
        History is stored for the configured retention period (default 4 weeks).

        Parameters
        ----------
        filter : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter operation execution history by this query.
            Available filters:

            - `id[eq]` - Filter by specific operation ID
            - `integration_id[eq]` - Filter by specific integration ID
            - `account_id[eq]` - Filter by specific account ID
            - `execution_id[eq]` - Filter by specific execution ID
            - `started_at[gte]` - Filter executions started at or after a specific datetime (RFC3339 format)
            - `started_at[lte]` - Filter executions started at or before a specific datetime (RFC3339 format)
            - `started_at[gt]` - Filter executions started after a specific datetime
            - `started_at[lt]` - Filter executions started before a specific datetime
            - `operation_id[eq]` - Filter by operation name (e.g., "assets_query_devices")
              If used more than once, the queries are ANDed together.

        order : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Select a field to order the results by. Defaults to `started_at[desc]`. To control the direction of the sorting, append
            `[asc]` or `[desc]` to the field id. For example, `started_at[asc]` will sort the results by `started_at` in ascending order.
            The ordering defaults to `desc` if not specified. May be used multiple times to order by multiple fields, and the
            ordering is applied in the order the fields are specified.

        limit : typing.Optional[int]
            Number of results to return (default 100, max 500)

        start_after : typing.Optional[str]
            Return execution history starting after this cursor.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListExecutionHistoryResponse

        Examples
        --------
        import asyncio

        from synqly import AsyncSynqlyManagement

        client = AsyncSynqlyManagement(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.operations.list_execution_history(
                filter="string",
                order="string",
                limit=1,
                start_after="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/jobs/history",
            method="GET",
            params={
                "filter": filter,
                "order": order,
                "limit": limit,
                "start_after": start_after,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    ListExecutionHistoryResponse,
                    construct_type(
                        type_=ListExecutionHistoryResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        Problem,
                        construct_type(
                            type_=Problem,  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .query_cloud_resource_inventory_response import QueryCloudResourceInventoryResponse
from .query_cloud_resource_inventory_response_generic import (
    QueryCloudResourceInventoryResponseGeneric,
)
from .query_cloud_security_threats_response import QueryCloudSecurityThreatsResponse
from .query_cloud_security_threats_response_generic import (
    QueryCloudSecurityThreatsResponseGeneric,
)
from .query_compliance_findings_response import QueryComplianceFindingsResponse
from .query_compliance_findings_response_generic import (
    QueryComplianceFindingsResponseGeneric,
)
from .query_events_response import QueryEventsResponse
from .query_events_response_generic import QueryEventsResponseGeneric
from .query_ioms_response import QueryIomsResponse
from .query_ioms_response_generic import QueryIomsResponseGeneric

__all__ = [
    "QueryCloudResourceInventoryResponse",
    "QueryCloudResourceInventoryResponseGeneric",
    "QueryCloudSecurityThreatsResponse",
    "QueryCloudSecurityThreatsResponseGeneric",
    "QueryComplianceFindingsResponse",
    "QueryComplianceFindingsResponseGeneric",
    "QueryEventsResponse",
    "QueryEventsResponseGeneric",
    "QueryIomsResponse",
    "QueryIomsResponseGeneric",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ....core.unchecked_base_model import UncheckedBaseModel
import typing
from .id import Id
import pydantic
import datetime as dt
from .external_reference import ExternalReference
from .marking_definition import MarkingDefinition
from .granular_marking import GranularMarking
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.pydantic_utilities import update_forward_refs


class CommonProperties(UncheckedBaseModel):
    spec_version: typing.Literal["2.1"] = "2.1"
    id: Id = pydantic.Field()
    """
    The id property uniquely identifies this object.
    """

    created_by_ref: typing.Optional["Identity"] = pydantic.Field(default=None)
    """
    The created_by_ref property specifies the id property of the identity object that describes the entity that created this object.
    """

    created: dt.datetime = pydantic.Field()
    """
    The created property represents the time at which the object was originally created.
    """

    modified: dt.datetime = pydantic.Field()
    """
    The modified property is only used by STIX Objects that support versioning and represents the time that this particular version of the object was last modified.
    """

    revoked: typing.Optional[bool] = pydantic.Field(default=None)
    """
    The revoked property indicates whether this object has been revoked. If the revoked property is present and set to true, then the object has been revoked.
    """

    labels: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The labels property is an array of strings that are used to categorize this object.
    """

    confidence: typing.Optional[int] = pydantic.Field(default=None)
    """
    The confidence property is an integer from 0 to 100 that represents the confidence that this object is accurate and valid.
    """

    lang: typing.Optional[str] = pydantic.Field(default=None)
    """
    The lang property specifies the language used in the properties of the object.
    """

    external_references: typing.Optional[typing.List[ExternalReference]] = (
        pydantic.Field(default=None)
    )
    """
    The external_references property is an array of external references that are relevant to this object.
    """

    object_marking_refs: typing.Optional[typing.List[MarkingDefinition]] = (
        pydantic.Field(default=None)
    )
    """
    The object_marking_refs property is an array of ids of marking_definition objects that apply to this object.
    """

    granular_markings: typing.Optional[typing.List[GranularMarking]] = pydantic.Field(
        default=None
    )
    """
    The granular_markings property is an array of granular markings that are applied to this object.
    """

    extensions: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = (
        pydantic.Field(default=None)
    )
    """
    The extensions property is an object that contains custom properties or objects that extend the object.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow


from .identity import Identity  # noqa: E402

update_forward_refs(CommonProperties)



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
import datetime as dt
from .incident_response_escalation_rule import IncidentResponseEscalationRule
from .incident_response_team_union import IncidentResponseTeamUnion
from ...common.types.id import Id
from ...engine.types.object import Object
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class IncidentResponseEscalationPolicy(UncheckedBaseModel):
    created_time: typing.Optional[int] = pydantic.Field(default=None)
    """
    A Unix timestamp in milliseconds representing the time the escalation policy was created.
    """

    created_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The human readable time the escalation policy was created.
    """

    desc: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the escalation policy.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the escalation policy.
    """

    escalation_rules: typing.Optional[typing.List[IncidentResponseEscalationRule]] = (
        pydantic.Field(default=None)
    )
    """
    The escalation rules for the escalation policy.
    """

    src_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    A deep-linked url to the escalation policy in a providers interface.
    """

    teams: typing.Optional[typing.List[IncidentResponseTeamUnion]] = pydantic.Field(
        default=None
    )
    """
    The teams that own the escalation policy.
    """

    uid: Id = pydantic.Field()
    """
    The unique identifier for the escalation policy.
    """

    updated_time: typing.Optional[int] = pydantic.Field(default=None)
    """
    A Unix timestamp in milliseconds representing the time the escalation policy was last updated.
    """

    updated_time_dt: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The human readable time the escalation policy was last updated.
    """

    unmapped: typing.Optional[Object] = pydantic.Field(default=None)
    """
    The attributes that are not mapped to the incident response user schema. The names and values of those attributes are specific to each provider.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .create_integration_token_request import CreateIntegrationTokenRequest
from .create_integration_token_response import CreateIntegrationTokenResponse
from .create_mcp_token_request import CreateMcpTokenRequest
from .create_mcp_token_response import CreateMcpTokenResponse
from .create_synqly_integrations_token_request import (
    CreateSynqlyIntegrationsTokenRequest,
)
from .create_synqly_integrations_token_response import (
    CreateSynqlyIntegrationsTokenResponse,
)
from .create_token_request import CreateTokenRequest
from .create_token_response import CreateTokenResponse
from .get_token_response import GetTokenResponse
from .list_tokens_response import ListTokensResponse
from .mcp_developer_usage_scope_options import McpDeveloperUsageScopeOptions
from .mcp_integration_usage_scope_options import McpIntegrationUsageScopeOptions
from .mcp_management_scope_options import McpManagementScopeOptions
from .mcp_token_scope import (
    McpTokenScope,
    McpTokenScope_Developer,
    McpTokenScope_IntegrationUsage,
    McpTokenScope_Management,
)
from .refresh_token import RefreshToken
from .refresh_token_response import RefreshTokenResponse
from .reset_token_response import ResetTokenResponse
from .rotate_token_response import RotateTokenResponse
from .synqly_integrations_token_response import SynqlyIntegrationsTokenResponse

__all__ = [
    "CreateIntegrationTokenRequest",
    "CreateIntegrationTokenResponse",
    "CreateMcpTokenRequest",
    "CreateMcpTokenResponse",
    "CreateSynqlyIntegrationsTokenRequest",
    "CreateSynqlyIntegrationsTokenResponse",
    "CreateTokenRequest",
    "CreateTokenResponse",
    "GetTokenResponse",
    "ListTokensResponse",
    "McpDeveloperUsageScopeOptions",
    "McpIntegrationUsageScopeOptions",
    "McpManagementScopeOptions",
    "McpTokenScope",
    "McpTokenScope_Developer",
    "McpTokenScope_IntegrationUsage",
    "McpTokenScope_Management",
    "RefreshToken",
    "RefreshTokenResponse",
    "ResetTokenResponse",
    "RotateTokenResponse",
    "SynqlyIntegrationsTokenResponse",
]



# This file was auto-generated by Fern from our API Definition.

from .async_operation_request_id import AsyncOperationRequestId
from .operation import Operation
from .operation_error import OperationError
from .operation_input import OperationInput
from .operation_status import OperationStatus

__all__ = [
    "AsyncOperationRequestId",
    "Operation",
    "OperationError",
    "OperationInput",
    "OperationStatus",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
from .open_text_core_application_security_credential import (
    OpenTextCoreApplicationSecurityCredential,
)
import pydantic
from .open_text_core_application_security_url import OpenTextCoreApplicationSecurityUrl
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class AppsecOpenTextCoreApplicationSecurity(UncheckedBaseModel):
    """
    Configuration for OpenText Core Application Security (formerly Fortify On Demand) as an application security provider.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/opentext-core-applicationsecurity-appsec-setup)
    """

    credential: OpenTextCoreApplicationSecurityCredential = pydantic.Field()
    """
    Credentials used for accessing the OpenText Core Application Security API.
    """

    url: OpenTextCoreApplicationSecurityUrl = pydantic.Field()
    """
    Base URL for the OpenText Core Application Security API. This URL should be the same as the URL used to access the OpenText Core Application Security web interface.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .builtin_roles import BuiltinRoles
from .create_role_request import CreateRoleRequest
from .create_role_response import CreateRoleResponse
from .get_role_response import GetRoleResponse
from .list_roles_response import ListRolesResponse
from .patch_role_response import PatchRoleResponse
from .role_definition import RoleDefinition
from .update_role_request import UpdateRoleRequest
from .update_role_response import UpdateRoleResponse

__all__ = [
    "BuiltinRoles",
    "CreateRoleRequest",
    "CreateRoleResponse",
    "GetRoleResponse",
    "ListRolesResponse",
    "PatchRoleResponse",
    "RoleDefinition",
    "UpdateRoleRequest",
    "UpdateRoleResponse",
]



# This file was auto-generated by Fern from our API Definition.

from ...core.unchecked_base_model import UncheckedBaseModel
import typing
from .splunk_hec_token import SplunkHecToken
import pydantic
from .splunk_search_credential import SplunkSearchCredential
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class SiemSplunk(UncheckedBaseModel):
    """
    Configuration for Splunk Enterprise Security.

    [Configuration guide](https://docs.synqly.com/guides/provider-configuration/splunk-setup)
    """

    hec_credential: typing.Optional[SplunkHecToken] = pydantic.Field(default=None)
    """
    Credential to use when connecting to the Splunk HEC service. If not provided, sending events to Splunk is disabled.
    """

    hec_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the Splunk HEC endpoint. Must include the full path to the HEC endpoint.
    """

    index: typing.Optional[str] = pydantic.Field(default=None)
    """
    Splunk index to send events to. If not provided, will use the default index for the Splunk collector.
    """

    search_service_credential: SplunkSearchCredential = pydantic.Field()
    """
    Credential used when authenticating with the Splunk Search Service.
    """

    search_service_url: str = pydantic.Field()
    """
    URL used when connecting to the Splunk Search Service.
    """

    skip_tls_verify: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When true, skips verification of the Splunk TLS certificate.
    """

    source: typing.Optional[str] = pydantic.Field(default=None)
    """
    Splunk source to send events to. If not provided the default source for the Splunk collector is used.
    """

    source_type: typing.Optional[str] = pydantic.Field(default=None)
    """
    Splunk source type to send events to. If not provided the default source type for the Splunk collector is used.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(
            extra="allow"
        )  # type: ignore # Pydantic v2
    else:

        class Config:
            smart_union = True
            extra = pydantic.Extra.allow
